--- old/usr/src/common/ctf/ctf_create.c	Wed Jan 29 09:35:13 2020
+++ new/usr/src/common/ctf/ctf_create.c	Wed Jan 29 09:35:13 2020
@@ -25,7 +25,7 @@
  * Use is subject to license terms.
  */
 /*
- * Copyright (c) 2019, Joyent, Inc.
+ * Copyright 2020 Joyent, Inc.
  */
 
 #include <sys/sysmacros.h>
@@ -1246,8 +1246,12 @@
 	return (type);
 }
 
+/*
+ * If size is 0, we use the standard integer size. This is almost always the
+ * case, except for packed enums.
+ */
 ctf_id_t
-ctf_add_enum(ctf_file_t *fp, uint_t flag, const char *name)
+ctf_add_enum(ctf_file_t *fp, uint_t flag, const char *name, size_t size)
 {
 	ctf_hash_t *hp = &fp->ctf_enums;
 	ctf_helem_t *hep = NULL;
@@ -1272,8 +1276,18 @@
 
 	VERIFY(type != CTF_ERR);
 	dtd->dtd_data.ctt_info = CTF_TYPE_INFO(CTF_K_ENUM, flag, 0);
-	dtd->dtd_data.ctt_size = fp->ctf_dmodel->ctd_int;
 
+	if (size == 0) {
+		dtd->dtd_data.ctt_size = fp->ctf_dmodel->ctd_int;
+	} else {
+		if (size > CTF_MAX_SIZE) {
+			dtd->dtd_data.ctt_size = CTF_LSIZE_SENT;
+			dtd->dtd_data.ctt_lsizehi = CTF_SIZE_TO_LSIZE_HI(size);
+			dtd->dtd_data.ctt_lsizelo = CTF_SIZE_TO_LSIZE_LO(size);
+		} else
+			dtd->dtd_data.ctt_size = size;
+	}
+
 	/*
 	 * Always dirty in case we modified a forward.
 	 */
@@ -1928,7 +1942,8 @@
 			    ctf_enum_iter(dst_fp, dst_type, enumcmp, &src))
 				return (ctf_set_errno(dst_fp, ECTF_CONFLICT));
 		} else {
-			dst_type = ctf_add_enum(dst_fp, flag, name);
+			size_t size = ctf_type_size(src_fp, src_type);
+			dst_type = ctf_add_enum(dst_fp, flag, name, size);
 			if ((dst.ctb_type = dst_type) == CTF_ERR ||
 			    ctf_enum_iter(src_fp, src_type, enumadd, &dst))
 				return (CTF_ERR); /* errno is set for us */
